<?php
/**
 * Created by JetBrains PhpStorm.
 * User: erikasouza
 * Date: 22/10/12
 * Time: 10:01
 * To change this template use File | Settings | File Templates.
 */

namespace Estoque\Logic\FechamentoDia;

use Estoque\Entities\Posestdia;
use Estoque\Entities\Inverota;
use Estoque\Logic\FechamentoDia\FechamentoDia;
use Estoque\Logic\FechamentoDia\Inventario;
use Estoque\Util\Parametros;
use \Geral\Util\DateUtil;
use \Estoque\Logic\Exception\ExceptionFechamentoDia;

use Geral\Entities\Diafecmes;
use Geral\Entities\Empresa;
use Geral\Entities\Filial;
use Geral\Entities\Localestoque;
use Geral\Entities\Paramfilial;
use Geral\Entities\Paramintegra;

use Fiscal\Entities\Notafiscal;

use Suprimentos\Entities\Prsubgrprod;

use Indeterminado\Entities\Receitabruta;

use Setforms2\Runtime\Helper\Procedure;
use Setforms2\Runtime\Helper\Procedure\Param;
use Setforms2\Runtime\Helper\Report\QR2;

class FechamentoMes{


    CONST FORMATEDATE_EN = 'y-MM-dd';
    CONST FORMATEDATE = 'Y-m-d H:m:s';


    private $realizaCalculo;

    private $geraInventRotaPosicao;
    private $utilInventRotaRequisicao;
    private $inventRealizado;

    private $tipoInvent;
    private $percentual;


    public function __construct(){
        /* Parametros Fechamento Mes */
        $this->realizaCalculo = Parametros::getCalculoReceitaBruta();
        $this->geraInventRotaPosicao = Parametros::getGeraInventarioRotativo();
        $this->utilInventRotaRequisicao = Parametros::getInventarioRotativoRequisicao();
        $this->inventRealizado = Parametros::getInventarioEstoqueCego();

        $this->tipoInvent = Parametros::getTipoInventarioViradaMes();
        $this->percentual = Parametros::getPercentualInventarioRotativo();
    }


    /**
     * Retorna a data o fechamento do mes.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtproc
     *
     * @return bool|\DateTime|string
     */
    public function consultaDataFechMes($nrorg, $filial, $dtproc){
        $dtFecham = false;

        $primDia = DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataDeString($dtproc));
        $ultDia  = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($dtproc));

        /* @var $diaFecMesRepository \Geral\Repositories\Diafecmes*/
        $diaFecMesRepository = Diafecmes::getRepository();

        /* @var $dados Diafecmes */
        $dados = $diaFecMesRepository->consultaDataFechMes(
            $nrorg, $filial, $primDia, $ultDia
        );

        if ($dados === false) {
            $diaFecham = Parametros::getDiaFechamentoMes();
            if (empty($diaFecham) === false) {
                $mesAnoFecham = DateUtil::getDataDeString($dtproc)->format("/m/Y");
                $dtFecham     = $diaFecham.$mesAnoFecham;
            }
        } else {
            $dtFecham = $dados->getDtfecmes();
        }

        return $dtFecham;
    }


    /**
     * Valida se a proxima data de processamento é a última oportunidade para o fechamento do mês, considerando
     * feriados e data do fechamento do mês.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProc
     * @param $dtProxDtProces
     *
     * @return bool|\DateTime|string
     */
    public function validaDiaFechamentoMes($nrorg, $filial, $dtProc, $dtProxDtProces){
        $dtFechMes = self::consultaDataFechMes($nrorg, $filial, $dtProc);
        $dtAntProc = DateUtil::subtraiIntervalo(DateUtil::getDataDeString($dtProxDtProces), 1);

        $util = new Util();
        $dtProxProcFeriado = $util->verificaFeriado($filial, $nrorg, $dtAntProc);

        $validaDiaFechMes = ($dataProc = $dtFechMes OR
            ($dtFechMes < $dtProxDtProces AND $dtProxProcFeriado !== false AND
                $dtFechMes <= $dtProxProcFeriado AND $dtFechMes > $dataProc));

        return $validaDiaFechMes;
    }


    /**
     * Verifica se há produtos irregulares.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProcesso
     * @param $dtProxDtProces
     *
     * @return bool
     */
    public static function verificaPosicaoEstoque($nrorg, $filial, $dtProcesso, $dtProxDtProces){
        $validaDiaFechMes = FechamentoMes::validaDiaFechamentoMes($nrorg, $filial, $dtProcesso, $dtProxDtProces);
        if ($validaDiaFechMes === true) {
            $dtFinal = DateUtil::adicionaIntervalo(DateUtil::getDataDeString($dtProcesso));
            $dtFinal = $dtFinal->format(DateUtil::FORMATO_BRASILEIRO);

            $dtInicial = DateUtil::subtraiIntervalo(DateUtil::getDataDeString($dtProcesso), 31);
            $dtInicial = $dtInicial->format(DateUtil::FORMATO_BRASILEIRO);

            /* @var $posestdiaRepository \Estoque\Repositories\Posestdia*/
            $posestdiaRepository = Posestdia::getRepository();
            $produtoIrregular = $posestdiaRepository->verificaProdutoIrregular($nrorg, $filial, $dtInicial, $dtFinal);
            if ($produtoIrregular) {
                \Estoque\Logic\Exception\ExceptionFechamentoDia::erroProdutoIrregular();
            }
        }

        return true;
    }


    /**
     * Parte III do Fechamento do dia. (Se for dia do Fechamento do Mês.)
     *
     * @param $nrorg
     * @param $filial
     * @param $operador
     * @param $dtProc
     * @param $dtProxDtProces
     * @param $utilAlmox
     * @param $utilLote
     * @param $cdAlmox
     * @param $cdLocal
     * @param $realizaInventDia
     *
     * @return bool
     *
     * @throws \Exception
     */
    public function transacaoFechamentoMes(
        $nrorg, $filial, $operador, $dtProc, $dtProxDtProces, $utilAlmox,
        $utilLote, $cdAlmox, $cdLocal, $realizaInventDia)
    {
        /** @var $objFilial Filial */
        $objFilial = Filial::getObject(array(
            'cdfilial' => $filial,
            'nrorg'    => $nrorg,
        ));

        $empresa = $objFilial->getEmpresa()->getCdempresa();

        /** @var $objEmpresa Empresa */
        $objEmpresa = Empresa::getObject(array(
            'cdempresa' => $empresa,
            'nrorg'     => $nrorg,
        ));

        $dataAtual = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);

        $util  = new Util();
        /** @var $datas \Estoque\Logic\FechamentoDia\Util */
        $datas = $util->primeiroUltimoDiaMes($dtProc);

        $inventario = new Inventario();
        $diaFechMes = $this->validaDiaFechamentoMes($nrorg, $filial, $dtProc, $dtProxDtProces);
        if ($diaFechMes === true) {
            self::calculoSimplesNacional(
                $this->realizaCalculo, $nrorg, $empresa, $objEmpresa, $datas->primeiroDiaMes, $datas->ultimoDiaMes,
                $dtProc, $operador, $dataAtual
            );

            self::atualizaDataReceitaNF($empresa, $datas->primeiroDiaMes, $datas->ultimoDiaMes, $nrorg, $dtProc);

            if ($utilAlmox === 'S') {
                $inventario->calculoInventarioEstoque(
                    $nrorg, $filial, $dtProc, $utilLote, $dataAtual, $operador, $realizaInventDia,
                    $this->geraInventRotaPosicao, $this->utilInventRotaRequisicao, $this->inventRealizado,
                    $this->tipoInvent, $this->percentual
                );
            }

            $inventario->calculoInventarioFiscalMes($nrorg, $filial, $dtProc, $operador, $dataAtual);
            $inventario->inventarioPosicaoEstoque(
                $nrorg, $filial, $dtProc, $utilLote, $dataAtual, $operador, $cdAlmox, $cdLocal
            );

            $inventario->inventarioRequisicao (
                $nrorg, $filial, $objFilial, $dataAtual, $dtProc, $cdAlmox, $cdLocal,
                $utilLote, $realizaInventDia
            );

            $this->configIntegracao($nrorg, $filial, $dtProc);
        }

        $fechamentoDia = new FechamentoDia($dtProc);
        $fechamentoDia->validaRE($filial, $nrorg, $dtProc);

        return true;
    }


    /**
     * Realiza o calculo da receita bruta de uma determinada empresa.
     *
     * @param $realizaCalculo
     * @param $nrorg
     * @param $empresa
     * @param \Geral\Entities\Empresa $objEmpresa
     * @param $primeiro
     * @param $ultimo
     * @param $dtProc
     * @param $operador
     * @param $dtAtual
     *
     * @return bool
     */
    public static function calculoSimplesNacional(
        $realizaCalculo, $nrorg, $empresa, Empresa $objEmpresa, $primeiro, $ultimo, $dtProc, $operador, $dtAtual
    ){
        if ($realizaCalculo === 'S') {
            /** @var $receitaBrutaRepository \Indeterminado\Repositories\Receitabruta */
            $receitaBrutaRepository = Receitabruta::getRepository();
            $receita = $receitaBrutaRepository->consultaReceitaBrutaEmpresa($nrorg, $empresa, $dtProc);

            try {
                if ($receita === false) {
                    $receitaBrutaRepository->atualizaReceita($empresa, $primeiro, $ultimo, $nrorg);
                } else {
//                    $dataProc  = new \DateTime($dtProc);
//                    $dataAtual = new \DateTime($dtAtual);
                    $dataProc  = DateUtil::getDataDeString($dtProc);
                    $dataAtual = DateUtil::getDataDeString($dtAtual);

                    $receitaBruta = new Receitabruta();

                    $receitaBruta->setEmpresa($objEmpresa);
                    $receitaBruta->setDtrefreceitabr($dataProc);
                    $receitaBruta->setIdstatusmes('B');
                    $receitaBruta->setVrreceitabr(0);
                    $receitaBruta->setVrsaldonegmes(0);
                    $receitaBruta->setVrsaldocompoe(0);
                    $receitaBruta->setVrsaldodeduz(0);
                    $receitaBruta->setIdinfmanual('M');
                    $receitaBruta->setIdstatusmes('P');
                    $receitaBruta->setCdoperinclusao($operador);
                    $receitaBruta->setCdoperultatu($operador);
                    $receitaBruta->setDtinclusao($dataAtual);
                    $receitaBruta->setDtultatu($dataAtual);
                    $receitaBruta->setNrorginclusao($nrorg);
                    $receitaBruta->setNrorgultatu($nrorg);

                    $receitaBruta->persist();
                }
            } catch (\Estoque\Logic\Exception\ExceptionFechamentoDia $e) {
                ExceptionFechamentoDia::erroCalculoSimples();
            }
        }

        return true;
    }


    /**
     * Atualiza a data do calculo da receita bruta da nota fiscal.
     *
     * @param $empresa
     * @param $primeiroDiaMesDtProc
     * @param $ultimoDiaMesDtProc
     * @param $nrorg
     * @param $dtProc
     *
     * @return bool
     */
    public static function atualizaDataReceitaNF($empresa, $primeiroDiaMesDtProc, $ultimoDiaMesDtProc, $nrorg, $dtProc){
        /** @var $notafiscalRepository \Fiscal\Repositories\Notafiscal */
        $notafiscalRepository = Notafiscal::getRepository();
        $notas = $notafiscalRepository->consultaNotasPorTipoOperador(
            $empresa, $primeiroDiaMesDtProc, $ultimoDiaMesDtProc, $nrorg
        );

        if ($notas) {
            try {
                $data = DateUtil::getDataDeString($dtProc);

                /** @var $nota \Fiscal\Entities\Notafiscal */
                foreach ($notas as $nota) {
                    /** @var $objNF \Fiscal\Entities\Notafiscal */
                    $objNF = Notafiscal::getObject(
                        array(
                            'nrorg' => $nrorg,
                            'cdfilial' => $nota->getFilial()->getCdfilial(),
                            'nrlanctonf' => $nota->getNrlanctonf(),
                        )
                    );

                    $objNF->atualizaDataNF($data);
                }
            } catch (\Estoque\Logic\Exception\ExceptionFechamentoDia $e) {
                ExceptionFechamentoDia::erroAtualizaDtReceita();
            }
        }

        return true;
    }


    public function configIntegracao($nrorg, $filial, $dtProc){
        /** @var $paramsIntegracao Paramintegra */
        $paramsIntegracao = Paramintegra::getObjectByWhere('NRORG = ?', array($nrorg));

        if ($paramsIntegracao) {
            if ($paramsIntegracao->getIdemissinvfmes() === 'S') {
                $nomeProcedure = $paramsIntegracao->getDsproceinvfmes();
                if (empty($nomeProcedure) === false) {
                    $dtFech = self::consultaDataFechMes($nrorg, $filial, $dtProc);

                    $procedure = new Procedure($nomeProcedure);

                    $procedure->addParam(new Param(
                        "P_CDFILIAL",
                        Param::PARAM_INPUT,
                        $filial,
                        Param::PARAM_TYPE_STR
                    ));
                    $procedure->addParam(new Param(
                        "P_DATA",
                        Param::PARAM_INPUT,
                        $dtFech,
                        Param::PARAM_TYPE_STR
                    ));

                    $procedure->call();

                    $diretorio = $paramsIntegracao->getDsdiretinvfmes();
                    if ($diretorio) {
                        $grupo = $paramsIntegracao->getGrproinvfmes()->getCdgrupprod();
                        if (empty($grupo) === true) {
                            $grupo = 'T';
                        }

                        /** @var $dadosFilialSubGrupo \Suprimentos\Entities\Prsubgrprod */
                        $dadosFilialSubGrupo = Prsubgrprod::getObjectByWhere(
                            'NRORG = ? AND CDGRUPPROD = ? AND CDFILISUBGR = ?',
                            array($nrorg, $grupo, $filial),
                            'CDFILISUBGR ASC'
                        );

                        $filialSubGrupo = $dadosFilialSubGrupo->getCdfilisubgr();
                        if (empty($filialSubGrupo) === true) {
                            $filialSubGrupo = 'P';
                        }

                        $params = array(
                            "P_FILIAL" => $filial,
                            "P_DTPOSICAO" => $dtFech,
                            "P_DTCUSTPROD" => $dtFech,
                            "P_DATAPOS" => $dtFech,
                            "P_TIPOCUSTO" => $paramsIntegracao->getTpcusinvfmes()->getCdtipocusto(),
                            "P_DESCRICAO" => $paramsIntegracao->getDsmensainvfmes(),
                            "P_CDGRUPPROD" => $grupo,
                            "P_CDFILISUBGR" => $filialSubGrupo,
                        );

                        $nmArqOrig = 'GRS03303.QR2';

                        $Qr2 = new QR2();
                        $Qr2->openReport($nmArqOrig, $params);

                        $dirOrig = realpath(dirname(__FILE__).'../../source/tecfood/public/reports');

                        list($dia, $mes, $ano) = explode('-', $dtFech);
                        $nomePdf = "Inventário de Fechamento do Mês ".$filial.$mes.$ano.".pdf";
                        $nomePdfNovo = strtr(utf8_decode($nomePdf), utf8_decode("áàãâéêíóôõúüçÁÀÃÂÉÊÍÓÔÕÚÜÇ "), "aaaaeeiooouucAAAAEEIOOOUUC_");
                        $nomePdf = utf8_encode($nomePdfNovo);

                        $copias = $paramsIntegracao->getNrcopiainvfmes();
                        rename($dirOrig.DIRECTORY_SEPARATOR.$nmArqOrig, $diretorio.DIRECTORY_SEPARATOR.$nomePdf);
                        $nomeArray = explode('.', $nomePdf);
                        $nome = $nomeArray[0];
                        $extensao = $nomeArray[1];
                        do {
                            copy($diretorio.DIRECTORY_SEPARATOR.$nomePdf, $diretorio.DIRECTORY_SEPARATOR.$nome.'_Copia'.$copias.'.'.$extensao);
                            $copias--;
                        } while ($copias > 0);
                    }
                }
            }
        }

        return true;
    }


}