<?php


/**
 * CNABSantander.
 *
 * @package Logic
 *
 * @author  thalesvalias
 */
class CNABSantander{

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_mesCompetencia;
    private $_dtInicio;
    private $_dtFim;
    private $_dtAux;
    private $_estrutura;
    private $_movimentacao;
    private $_estruturacao;
    private $_tpMovTransfM;
    private $_tpMovTransfH;
    private $_itemGrupo;
    private $_listaCalculoFolha;
    private $_evento;

    private $_registro0;
    private $_registro1;
    private $_registro3A;
    private $_registro3B;
    private $_registro5;
    private $_registro9;
    private $_cnabMagnetico;
    private $_totalVinculos; // Total de vinculos por estrutura.
    private $_dadoVinculo = array();
    private $_cnabTeste;
    private $_codConvenio;

    private $_itemCalculo;
    private $_pessoa;
    private $_vinculo;
    private $_vinculoEmpreg;
    private $_ocupacao;
    private $_tpDemissao;
    private $_grauInstrucao;
    private $_escalaTrabalho;
    private $_raca;
    private $_condicaoFisica;
    private $_cbo;
    private $_enderecoParc;

    private $_nrTpMovimentoFormulario;

    const CODIGO_BANCO_SANTANDER = '033';


    /**
     * Construtor.
     *
     * @param integer             $nrOrg          Numero da Organizacao.
     * @param integer             $nrOrgPadrao    Numero da Organizacao Padrao.
     * @param TkDateTime          $mesCompetencia Mes de competencia.
     * @param FpacnabmagnetEntity $cnabMagnetico  Instancia de FpacnabmagnetEntity.
     * @param FpaformcnabEntity   $formulario     Instancia de FpaformcnabEntity.
     *
     * @author thalesvalias - 27/07/2012
     */
    public function __construct(
        $nrOrg, $nrOrgPadrao, TkDateTime $mesCompetencia, FpacnabmagnetEntity $cnabMagnetico, FpaformcnabEntity $formulario
    ){

        $this->_nrOrg = $nrOrg;
        $this->_nrOrgPadrao = $nrOrgPadrao;
        $this->_mesCompetencia = $mesCompetencia;

        $this->_dtInicio = new TkDateTime($this->_mesCompetencia);
        $this->_dtFim = new TkDateTime($this->_mesCompetencia);
        $this->_dtFim->addMonth(1)->subDay(1);
        $this->_dtAux = new TkDateTime();

         $this->_itemCalculo = new FpaitecalcfolhaEntity();
        $this->_pessoa = new GpepessoaEntity(null, $this->_nrOrg, $this->_mesCompetencia);
        $this->_vinculo = new GpevinculomEntity();
        $this->_vinculoEmpreg = new GpevinculoempregEntity();
        $this->_ocupacao = new GpeOcupacaomEntity(null, $this->_nrOrg, $this->_mesCompetencia);
        $this->_tpDemissao = new FpaTpdemissaoEntity(null, $this->_nrOrgPadrao, $this->_nrOrg);
        $this->_movimentacao = new GpemovimentacaoEntity();
        $this->_tpMovTransfM = new GpetpmovtransfmEntity(
            null, $this->_nrOrgPadrao, $this->_nrOrg, $this->_mesCompetencia
        );
        $this->_tpMovTransfH = new GpetpmovtransfhEntity(
            null, $this->_nrOrgPadrao, $this->_nrOrg, $this->_mesCompetencia
        );
        $this->_grauInstrucao = new GpeGrauinstrEntity(null, $this->_nrOrg, $this->_nrOrgPadrao);
        $this->_escalaTrabalho = new GpeescalatrabmEntity();
        $this->_raca = new GpeRacapessoaEntity(null, $this->_nrOrgPadrao, $this->_nrOrg);
        $this->_condicaoFisica = new GpeCondfispesEntity(null, $this->_nrOrg, $this->_nrOrgPadrao);
        $this->_cbo = new GpeCboEntity();
        $this->_enderecoParc = new EnderecoparcEntity();

        $this->_evento = new FpaeventomEntity(null, $this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);
        $this->_estruturacao = new Estruturacao($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);
        $this->_estrutura = new EstruturamEntity(null, $this->_nrOrg, $this->_mesCompetencia);

        $this->_cnabMagnetico = $cnabMagnetico;

        $this->_registro0 = new Registro0CNABSantander();
        $this->_registro1 = new Registro1CNABSantander();
        $this->_registro3A = new Registro3ACNABSantander();
        $this->_registro3B = new Registro3BCNABSantander();
        $this->_registro5 = new Registro5CNABSantander();
        $this->_registro9 = new Registro9CNABSantander();

        $this->_inicializaVariaveis($formulario);
    }


    private function _inicializaVariaveis($formulario){

        $this->_totalVinculos = array();
        $this->_nrTpMovimentoFormulario = $formulario->getNrtipomoviment();
        $this->_cnabTeste = $formulario->getIdteste();
        $this->_codConvenio = $formulario->getCodconvenio();
    }

    
    /**
     * Cria os registros CNAB e armazena na base.
     *
     * @param EstruturamEntity  $estrutura Instancia de EstruturamEntity.
     * @param GpepessoaEntity   $pessoa    Instancia de GpepessoaEntity.
     * @param GpevinculomEntity $vinculo   Instancia de GpevinculomEntity.
     *
     * @return void
     *
     * @author thalesvalias - 02/08/2012
     */
    public function geraCNAB(EstruturamEntity $estrutura, GpepessoaEntity $pessoa, GpevinculomEntity $vinculo){
        $this->_estrutura = $estrutura;
        $this->_pessoa = $pessoa;
        $this->_vinculo = $vinculo;

        $this->_preparaRegistro0();
        $this->_preparaRegistro1();
        $this->_preparaRegistro3A();
        $this->_preparaRegistro3B();
        $this->_preparaRegistro5();
        $this->_preparaRegistro9();
    }


    /**
     * Prepara o registro tipo 0 - Registro Header de Arquivo.
     *
     * @return void
     *
     * @author thalesvalias - 02/08/2012
     */
    private function _preparaRegistro0() {

        $posicao = $this->_cnabMagnetico->procuraRegistroLista(
            $this->_estrutura->nrInscricao, Registro0CNABSantander::REGISTRO_0, self::CODIGO_BANCO_SANTANDER,
            $this->_nrTpMovimentoFormulario
        );

        if ($posicao === false) {

            // Verifica se o CNAB esta sendo gerado para teste para gerar o numero sequencial de acordo com o layout
            if ($this->_cnabTeste === FpaFormcnabEntity::IDTESTE_NAO) {

                // CNAB Real - Producao
                $ultimoSequencial = $this->_cnabMagnetico->retornaUltimoSequencialRegistro0(
                    $this->_mesCompetencia, $this->_nrOrg, self::CODIGO_BANCO_SANTANDER
                );
                if (empty($ultimoSequencial) === false) {
                    $sequencia = $ultimoSequencial;
                    $sequencia++;
                } else {
                    $sequencia = FPARegistro::CARACTER_ONZE;
                }
            } else {
                // CNAB Teste
                $sequencia = FPARegistro::CARACTER_UM;
            }

            $this->_registro0->setSequencia((string)$sequencia);
            $this->_registro0->setEstrutura($this->_estrutura);
            $this->_registro0->setCodConvenioBanco($this->_codConvenio);

            // Monta a linha registro 0.
            $this->_registro0->geraRegistro0();

            // Insere novo registro 0.
            $this->_cnabMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_mesCompetencia, Registro0CNABSantander::REGISTRO_0, $this->_estrutura->nrTpInscricao,
                $this->_estrutura->nrInscricao, $this->_registro0->getLinhaRegistro(), self::CODIGO_BANCO_SANTANDER,
                $this->_nrTpMovimentoFormulario, $sequencia
            );

            // Limpa o registro 0.
            $this->_registro0->setLinhaRegistro();
        }
    }


    /**
     * Prepara o registro tipo 1 - Registro Header de Lote.
     *
     * @return void
     *
     * @author thalesvalias - 02/08/2012
     */
    private function _preparaRegistro1() {

        $posicao = $this->_cnabMagnetico->procuraRegistroLista(
            $this->_estrutura->nrInscricao, Registro1CNABSantander::REGISTRO_1, self::CODIGO_BANCO_SANTANDER,
            $this->_nrTpMovimentoFormulario
        );

        if ($posicao === false) {

            $this->_registro1->setEstrutura($this->_estrutura);
            $this->_registro1->setCodConvenioBanco($this->_codConvenio);

            // Monta a linha registro 1.
            $this->_registro1->geraRegistro1();

            // Insere novo registro 1.
            $this->_cnabMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_mesCompetencia, Registro1CNABSantander::REGISTRO_1, $this->_estrutura->nrTpInscricao,
                $this->_estrutura->nrInscricao, $this->_registro1->getLinhaRegistro(), self::CODIGO_BANCO_SANTANDER,
                $this->_nrTpMovimentoFormulario
            );

            // Limpa o registro 1.
            $this->_registro1->setLinhaRegistro();
        }
    }


    /**
     * Prepara o registro tipo 3A - Registro Detalhe.
     *
     * @return void
     *
     * @author thalesvalias - 02/08/2012
     */
    private function _preparaRegistro3A() {

        $this->_registro3A->setPessoa($this->_pessoa);
        $this->_registro3A->setVinculo($this->_vinculo);
        $this->_registro3A->setCnpjEmpresa($this->_estrutura->nrInscricao);

        // Registro 3A.
        $this->_registro3A->geraRegistro3A();

        // Persiste registro linha 3A.
        $this->_cnabMagnetico->persisteRegistro(
            $this->_nrOrg, $this->_mesCompetencia, RegistroCNAB::REGISTRO_3, Registro3ACNABSantander::SEGMENTO_A,
            $this->_estrutura->nrTpInscricao, $this->_estrutura->nrInscricao, $this->_registro3A->getLinhaRegistro(),
            $this->_vinculo->getNrvinculom(), self::CODIGO_BANCO_SANTANDER, $this->_nrTpMovimentoFormulario,
            $this->_vinculo->valorPagamento, $this->_pessoa->nrTpInscricao, $this->_pessoa->historico->getNrcpfpessoa()
        );

        // Limpa o registro 3A.
        $this->_registro3A->setLinhaRegistro();
    }


    /**
     * Prepara o registro tipo 3B - Registro Detalhe.
     *
     * @return void
     *
     * @author thalesvalias - 02/08/2012
     */
    private function _preparaRegistro3B() {

        $this->_registro3B->setPessoa($this->_pessoa);
        $this->_registro3B->setVinculo($this->_vinculo);

        // Registro 3B.
        $this->_registro3B->geraRegistro3B();

        // Persiste registro linha 3B.
        $this->_cnabMagnetico->persisteRegistro(
            $this->_nrOrg, $this->_mesCompetencia, RegistroCNAB::REGISTRO_3, Registro3BCNABSantander::SEGMENTO_B,
            $this->_estrutura->nrTpInscricao, $this->_estrutura->nrInscricao, $this->_registro3B->getLinhaRegistro(),
            $this->_vinculo->getNrvinculom(), self::CODIGO_BANCO_SANTANDER, $this->_nrTpMovimentoFormulario,
            $this->_vinculo->valorPagamento, $this->_pessoa->nrTpInscricao, $this->_pessoa->historico->getNrcpfpessoa()
        );

        // Limpa o registro 3B.
        $this->_registro3B->setLinhaRegistro();
    }


    /**
     * Prepara o registro tipo 5 - Registro Trailer de Lote.
     *
     * @return void
     *
     * @author thalesvalias - 02/08/2012
     */
    private function _preparaRegistro5() {

        $posicao = $this->_cnabMagnetico->procuraRegistroLista(
            $this->_estrutura->nrInscricao, Registro5CNABSantander::REGISTRO_5, self::CODIGO_BANCO_SANTANDER,
            $this->_nrTpMovimentoFormulario
        );

        if ($posicao === false) {

            // Monta a linha registro 5.
            $this->_registro5->geraRegistro5();

            // Insere novo registro 5.
            $this->_cnabMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_mesCompetencia, Registro5CNABSantander::REGISTRO_5, $this->_estrutura->nrTpInscricao,
                $this->_estrutura->nrInscricao, $this->_registro5->getLinhaRegistro(), self::CODIGO_BANCO_SANTANDER,
                $this->_nrTpMovimentoFormulario
            );

            // Limpa o registro 5.
            $this->_registro5->setLinhaRegistro();
        }
    }


    /**
     * Prepara o registro tipo 9 - Registro Trailer de Arquivo.
     *
     * @return void
     *
     * @author thalesvalias - 02/08/2012
     */
    private function _preparaRegistro9() {

        $posicao = $this->_cnabMagnetico->procuraRegistroLista(
            $this->_estrutura->nrInscricao, Registro9CNABSantander::REGISTRO_9, self::CODIGO_BANCO_SANTANDER
            , $this->_nrTpMovimentoFormulario
        );

        if ($posicao === false) {

            // Monta a linha registro 9.
            $this->_registro9->geraRegistro9();

            // Insere novo registro 9.
            $this->_cnabMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_mesCompetencia, Registro9CNABSantander::REGISTRO_9, $this->_estrutura->nrTpInscricao,
                $this->_estrutura->nrInscricao, $this->_registro9->getLinhaRegistro(), self::CODIGO_BANCO_SANTANDER,
                $this->_nrTpMovimentoFormulario
            );

            // Limpa o registro 9.
            $this->_registro9->setLinhaRegistro();
        }
    }


    /**
     * Retorna linha do registro modificada(atualizada) com as consolidacoes de acordo com o tipo de registro.
     *
     * @param integer $tipoRegistro        Tipo do registro da linha.
     * @param string  $linha               Linha do registro.
     * @param float   $valorTotalPagamento Valor consolidado do pagamento de todos os funcionarios(registro 3A).
     *
     * @return string (linha modificada do registro)
     *
     * @author thalesvalias - 07/08/2012
     */
    public function modificaLinhasConsolidacaoSantander($tipoRegistro, $linha, $valorTotalPagamento){

        if ($tipoRegistro === Registro5CNABSantander::REGISTRO_5) {

            // Quantidade de Registros do Lote.
            $qtdRegistrosLote = $this->_cnabMagnetico->totalRegistrosPorTipos(
                $this->_mesCompetencia, $this->_nrOrg, RegistroCNAB::LISTA_TIPO_REGISTROS_LOTE,
                self::CODIGO_BANCO_SANTANDER, $this->_nrTpMovimentoFormulario
            );

            // Bloco 5 | 018-023 | Quantidade de Registros do Lote (Registros Tipo 1, 3, 5).
            $linha = $this->_registro5->alteraDado(
                $linha, (string)$qtdRegistrosLote, (18 -1), 6, Registro5CNABSantander::CARACTER_ZERO, Registro5CNABSantander::PREENCHE_ESQUERDA
            );

            // Valor obtido pela somatoria dos valores de Creditos / Pagamentos dos registros de detalhe (Registros do Tipo 3 / Segmento A).
            $linha = $this->_registro5->alteraDado(
                $linha, $valorTotalPagamento, (24 -1), 18, Registro5CNABSantander::CARACTER_ZERO, Registro5CNABSantander::PREENCHE_ESQUERDA
            );

            // Bloco 7 | 042-059 | Somatoria Quantidade Moeda (Registro Tipo 3). Mesmo valor do bloco anterior (6).
            $linha = $this->_registro5->alteraDado(
                $linha, $valorTotalPagamento, (42 -1), 18, Registro5CNABSantander::CARACTER_ZERO, Registro5CNABSantander::PREENCHE_ESQUERDA
            );
        } elseif ($tipoRegistro === Registro9CNABSantander::REGISTRO_9) {

            // Quantidade de lotes do arquivo.
            $qtdLotesArquivo = $this->_cnabMagnetico->totalRegistrosPorTipos(
                $this->_mesCompetencia, $this->_nrOrg, RegistroCNAB::LISTA_TIPO_REGISTRO_CABECALHO_LOTE,
                self::CODIGO_BANCO_SANTANDER, $this->_nrTpMovimentoFormulario
            );

            // Quantidade de registros no arquivo.
            $qtdRegistrosArquivo = $this->_cnabMagnetico->totalRegistrosPorTipos(
                $this->_mesCompetencia, $this->_nrOrg, RegistroCNAB::LISTA_TIPO_REGISTROS_ARQUIVO,
                self::CODIGO_BANCO_SANTANDER, $this->_nrTpMovimentoFormulario
            );

            // Bloco 5 | 018-023 | Quantidade de lotes do arquivo (Registros Tipo 1).
            $linha = $this->_registro9->alteraDado(
                $linha, (string)$qtdLotesArquivo, (18 -1), 6, Registro9CNABSantander::CARACTER_ZERO, Registro9CNABSantander::PREENCHE_ESQUERDA
            );

            // Bloco 6 | 024-029 | Quantidade de registros no arquivo (Registros Tipo 0, 1, 3, 5 e 9).
            $linha = $this->_registro9->alteraDado(
                $linha, (string)$qtdRegistrosArquivo, (24 -1), 6, Registro9CNABSantander::CARACTER_ZERO, Registro9CNABSantander::PREENCHE_ESQUERDA
            );
        }

        return $linha;
    }

    
}